@@ -662,46 +662,50 @@ void fbComposeSetupMMX(void)
 
 /* ------------------ MMX code paths called from fbpict.c ----------------------- */
 
+typedef union {
+  __m64 m64;
+  uint64_t ull;
+} m64_ull;
 
 typedef struct
 {
-    ullong mmx_4x00ff;
-    ullong mmx_4x0080;
-    ullong mmx_565_rgb;
-    ullong mmx_565_unpack_multiplier;
-    ullong mmx_565_r;
-    ullong mmx_565_g;
-    ullong mmx_565_b;
-    ullong mmx_mask_0;
-    ullong mmx_mask_1;
-    ullong mmx_mask_2;
-    ullong mmx_mask_3;
-    ullong mmx_full_alpha;
-    ullong mmx_ffff0000ffff0000;
-    ullong mmx_0000ffff00000000;
-    ullong mmx_000000000000ffff;
+    m64_ull mmx_4x00ff;
+    m64_ull mmx_4x0080;
+    m64_ull mmx_565_rgb;
+    m64_ull mmx_565_unpack_multiplier;
+    m64_ull mmx_565_r;
+    m64_ull mmx_565_g;
+    m64_ull mmx_565_b;
+    m64_ull mmx_mask_0;
+    m64_ull mmx_mask_1;
+    m64_ull mmx_mask_2;
+    m64_ull mmx_mask_3;
+    m64_ull mmx_full_alpha;
+    m64_ull mmx_ffff0000ffff0000;
+    m64_ull mmx_0000ffff00000000;
+    m64_ull mmx_000000000000ffff;
 } MMXData;
 
 static const MMXData c =
 {
-    .mmx_4x00ff =			0x00ff00ff00ff00ffULL,
-    .mmx_4x0080 =			0x0080008000800080ULL,
-    .mmx_565_rgb =			0x000001f0003f001fULL,
-    .mmx_565_r =			0x000000f800000000ULL,
-    .mmx_565_g =			0x0000000000fc0000ULL,
-    .mmx_565_b =			0x00000000000000f8ULL,
-    .mmx_mask_0 =			0xffffffffffff0000ULL,
-    .mmx_mask_1 =			0xffffffff0000ffffULL,
-    .mmx_mask_2 =			0xffff0000ffffffffULL,
-    .mmx_mask_3 =			0x0000ffffffffffffULL,
-    .mmx_full_alpha =			0x00ff000000000000ULL,
-    .mmx_565_unpack_multiplier =	0x0000008404100840ULL,
-    .mmx_ffff0000ffff0000 =		0xffff0000ffff0000ULL,
-    .mmx_0000ffff00000000 =		0x0000ffff00000000ULL,
-    .mmx_000000000000ffff =		0x000000000000ffffULL,
+    .mmx_4x00ff.ull =			0x00ff00ff00ff00ffULL,
+    .mmx_4x0080.ull =			0x0080008000800080ULL,
+    .mmx_565_rgb.ull =			0x000001f0003f001fULL,
+    .mmx_565_r.ull =			0x000000f800000000ULL,
+    .mmx_565_g.ull =			0x0000000000fc0000ULL,
+    .mmx_565_b.ull =			0x00000000000000f8ULL,
+    .mmx_mask_0.ull =			0xffffffffffff0000ULL,
+    .mmx_mask_1.ull =			0xffffffff0000ffffULL,
+    .mmx_mask_2.ull =			0xffff0000ffffffffULL,
+    .mmx_mask_3.ull =			0x0000ffffffffffffULL,
+    .mmx_full_alpha.ull =			0x00ff000000000000ULL,
+    .mmx_565_unpack_multiplier.ull =	0x0000008404100840ULL,
+    .mmx_ffff0000ffff0000.ull =		0xffff0000ffff0000ULL,
+    .mmx_0000ffff00000000.ull =		0x0000ffff00000000ULL,
+    .mmx_000000000000ffff.ull =		0x000000000000ffffULL,
 };
 
-#define MC(x) ((__m64) c.mmx_##x)
+#define MC(x) ((__m64) c.mmx_##x.m64)
 
 static __inline__ __m64
 shift (__m64 v, int s)
@@ -726,8 +730,8 @@ pix_multiply (__m64 a, __m64 b)
     __m64 res;
     
     res = _mm_mullo_pi16 (a, b);
-    res = _mm_adds_pu16 (res, _mm_srli_pi16 (res, 8));
     res = _mm_adds_pu16 (res, MC(4x0080));
+    res = _mm_adds_pu16 (res, _mm_srli_pi16 (res, 8));
     res = _mm_srli_pi16 (res, 8);
     
     return res;
@@ -815,6 +819,12 @@ pack8888 (__m64 lo, __m64 hi)
     return r;
 }
 
+static __inline__ CARD32
+store8888 (__m64 v)
+{
+    return _mm_cvtsi64_si32(pack8888(v, _mm_setzero_si64()));
+}
+
 /* Expand 16 bits positioned at @pos (0-3) of a mmx register into
  *
  *    00RR00GG00BB
@@ -898,8 +908,7 @@ fbCompositeSolid_nx8888mmx (uint32_t *dst, uint32_t *src, int w)
 
     while (w && (unsigned long)dst & 7)
     {
-        *dst = (ullong) pack8888(over(vsrc, vsrca, load8888(*dst)),
-                                 _mm_setzero_si64());
+        *dst = store8888(over(vsrc, vsrca, load8888(*dst)));
         
         w--;
         dst++;
@@ -923,7 +932,7 @@ fbCompositeSolid_nx8888mmx (uint32_t *dst, uint32_t *src, int w)
     
     while (w)
     {
-        *dst = (ullong) pack8888(over(vsrc, vsrca, load8888(*dst)), _mm_setzero_si64());
+        *dst = store8888(over(vsrc, vsrca, load8888(*dst)));
         
         w--;
         dst++;
@@ -932,7 +941,7 @@ fbCompositeSolid_nx8888mmx (uint32_t *dst, uint32_t *src, int w)
     _mm_empty();
 }
 OIL_DEFINE_IMPL_FULL(fbCompositeSolid_nx8888mmx, composite_over_argb_const_src,
-    OIL_IMPL_FLAG_MMX);
+    OIL_IMPL_FLAG_MMX| OIL_IMPL_FLAG_MMXEXT);
 
 #if 0
 void
@@ -1910,7 +1919,7 @@ fbCompositeSrcAdd_8888x8888mmx (uint32_t *dst, uint32_t *src, int w)
     
     while (w >= 2)
     {
-        *(ullong*)dst = (ullong) _mm_adds_pu8(*(__m64*)src, *(__m64*)dst);
+        *(__m64 *)dst = _mm_adds_pu8(*(__m64*)src, *(__m64*)dst);
         dst += 2;
         src += 2;
         w -= 2;
