@@ -30,6 +30,11 @@
 
 #include <liboil/liboiltypes.h>
 
+/**
+ * OilType:
+ *
+ * Enumeration containing the data types understood by Liboil.
+ */
 typedef enum {
   OIL_TYPE_UNKNOWN = 0,
   OIL_TYPE_INT,
@@ -55,6 +60,12 @@ typedef enum {
   OIL_TYPE_f64p,
 } OilType;
 
+/**
+ * OilArgType:
+ *
+ * Enumeration containing the types of parameter types understood
+ * by Liboil.
+ */
 typedef enum {
   OIL_ARG_UNKNOWN = 0,
   OIL_ARG_N,
@@ -81,7 +92,14 @@ typedef enum {
   OIL_ARG_LAST
 } OilArgType;
 
+/**
+ * OilParameter:
+ * 
+ * An opaque structure representing a single parameter in the
+ * function prototype of an OilFunctionClass.
+ */
 struct _OilParameter {
+  /*< private >*/
   char *type_name;
   char *parameter_name;
 
@@ -109,7 +127,14 @@ struct _OilParameter {
   int stride;
   int size;
   int guard;
+  int test_header;
+  int test_footer;
 };
 
+void *oil_param_get_source_data (OilParameter *param);
+
+#define oil_type_is_floating_point(type) \
+  (((type) == OIL_TYPE_f64p) || ((type) == OIL_TYPE_f32p))
+
 #endif
 
